package MT::SheetAsset::CMS;

use strict;
use warnings;

use MT::Template::Context;
use MT::SheetAsset::Util;

sub cb_template_source_asset_chooser {
    return 1 unless installed;
    my ( $cb, $app, $rsource, $file ) = @_;

    my $header = '';
    my $types = $app->registry( 'sheetasset', 'types' ) || {};

    foreach my $key ( %$types ) {
        my $type = $types->{$key};
        my $label = $type->{label};
        $label = $label->() if ref $label eq 'CODE';
        $label ||= '';
        $header .= qq{
            <mt:if name="asset_type" eq="sheetasset_$key">
                <mt:setvar name="asset_type_label" value="$label" />
            </mt:if>
        };
    }

    my $footer = q{
        <mt:if name="asset_type" like="sheetasset_">
            <mt:unless name="simple">
                <mt:include name="asset_chooser_footer.tmpl" component="SheetAsset">
            </mt:unless>
        </mt:if>
    };


    $$rsource = $header . $$rsource . $footer;

    1;
}

sub cb_template_param_edit_asset {
    return 1 unless installed;
    my ( $cb, $app, $param, $tmpl ) = @_;

    # Check if SheetAsset
    my $asset_id = $param->{id} || return 1;
    my $asset = MT->model('asset')->load($asset_id);
    return 1
        if !$asset->can('is_sheetasset')
            || !$asset->is_sheetasset;

    # Screen Class
    $param->{screen_class} ||= '';
    $param->{screen_class} .= ' asset-sheetasset';

    # Ajax URI
    $param->{sheetasset_preview_uri} = $app->uri(
        mode => 'sheetasset_preview_table',
    );

    # Include library
    {
        my $target = $tmpl->getElementById('header_include');
        my $include = $tmpl->createElement('setvarblock', {
            name => 'js_include',
            append => 1,
        });
        $include->innerHTML(q{<mt:include name="preview_js_include.tmpl" component="SheetAsset" />});
        $tmpl->insertBefore($include, $target);
    }

    # Insert preview
    {
        my $target = $tmpl->getElementById('asset-url');
        my $insert = $tmpl->createElement('app:setting', {
            id => 'sheetasset_preview',
            label_class => 'top-label',
            label => '<__trans_section component="SheetAsset"><__trans phrase="Preview" /></__trans_section>',
        });
        $insert->innerHTML(q{
            <div
                class="sheetasset-preview"
                mt-blog-id="<mt:var name='blog_id' />"
                mt-asset-id="<mt:var name='id' />"
            ></div>
        });
        $tmpl->insertAfter($insert, $target);
    }

    1;
}

sub sheetasset_url_field_html_params {
    my ( $key, $tmpl_key, $tmpl_param ) = @_;

    require MT::SheetAsset::URLHandler::Base;
    my $handlers = MT::SheetAsset::URLHandler::Base->url_handlers;
    my @url_handlers;

    foreach my $h ( @$handlers ) {
        push @url_handlers, {
            label           => $h->label,
            instruction    => $h->instruction,
            example         => $h->example,
        };
    }

    $tmpl_param->{url_handlers} = \@url_handlers;

    1;
}

sub method_sheetasset_preview_table {
    return unless installed;
    my ( $app ) = @_;
    my $q = $app->param;

    # Requires valid blog context
    my $id = $q->param('id') || $q->param('asset_id');
    my $url = $q->param('url') || $q->param('sheet_url');

    my $blog;
    my $asset;
    if ( $id ) {
        # Requires Blog context
        return $app->json_error( plugin->tralsate('No blog context.') )
            unless $app->can('blog');
        $blog = $app->blog
            or return $app->json_error( plugin->translate('No blog context.') );
        $asset = MT->model('asset')->load({ id => $id, blog_id => $blog->id })
            or return $app->json_error( plugin->translate('No asset context.') );
    } elsif ( $url ) {
        my %ctx = ( url => $url );
        my $handler = url_handler(\%ctx)
            or return $app->json_error( plugin->translate('URL is not supported.') );
        $asset = $handler->download_asset(\%ctx)
            or return $app->json_error( $handler->errstr );
    }

    return $app->json_error( plugin->translate('No asset context.') )
        unless $asset;

    my $worksheet_index = $q->param('worksheet_index') || 0;
    return $app->json_error( plugin->translate('Not a sheet asset.') )
        unless $asset->can('sheetasset_workbook');

    my $wb = $asset->sheetasset_workbook
        or return $app->json_error( $asset->errstr );

    my $windowing = workbook_window_param;
    $wb->window($windowing);

    my $ws = $wb->worksheets->[$worksheet_index]
        or return $app->json_error( plugin->translate('No worksheet context.') );

    my $table_html = $ws->table_html({
        table_class => 'listing-table',
        column_class => 'col',
        header_class => 'col head',
    }) or return $app->json_error( $asset->errstr );

    my $i = 0;
    my @worksheets = map {
        my $url = $app->uri(
            mode => 'sheetasset_preview_table',
            args => {
                $blog ? ( blog_id => $blog->id ) : (),
                $id ? ( id => $id ) : (),
                $url ? ( url => $url ) : (),
                worksheet_index => $i,
            },
        );

        my $tupple = {
            index => $i,
            name => $_->name,
            url => $url,
            active => ($worksheet_index == $i),
        };
        $i++;

        $tupple;
    } @{$wb->worksheets};

    my $ctx  = MT::Template::Context->new;
    if ( $blog ) {
        $ctx->stash('blog', $blog);
        $ctx->stash('blog_id', $blog->id);
    }
    $ctx->stash('asset', $asset);
    $ctx->stash('asset_id', $asset->id);
    $ctx->var('sheetasset_table', $table_html);
    $ctx->var('sheetasset_active_sheet_index', $worksheet_index);
    $ctx->var('sheetasset_worksheets', \@worksheets)
        if @worksheets && ( scalar @worksheets > 1 || length($worksheets[0]->{name}) );
    $ctx->var('sheetasset_active_worksheets', [ $worksheets[$worksheet_index] ])
        if @worksheets && ( scalar @worksheets > 0 ) && $worksheets[$worksheet_index];

    foreach my $key ( %$windowing ) {
        next if $key eq 'worksheets' && !$ctx->var('sheetasset_worksheets');
        $ctx->var('sheetasset_window_' . $key, $windowing->{$key});
    }

    my $tmpl = plugin->load_tmpl('preview_table.tmpl');

    my %cond;
    defined ( my $html = $tmpl->build($ctx, \%cond) )
        or return $app->json_error($ctx->errstr);
    $html = plugin->translate_templatized($html);
    $app->json_result({ html => $html });
}

sub sheetasset_handsontable_field_html_params {
    my ( $key, $tmpl_key, $tmpl_param ) = @_;
    my $app = MT->instance;

    if ( $tmpl_key eq 'field_html' ) {
        my $system_config = plugin_config(0);
        $tmpl_param->{sheetasset_handsontable_system_opts_json} = $system_config->{sheetasset_handsontable_system_opts_json};

        if ( $app->blog ) {
            my $blog_config = plugin_config($app->blog->id);
            $tmpl_param->{sheetasset_handsontable_blog_opts_json} = $blog_config->{sheetasset_handsontable_blog_opts_json};
        }

        if ( !$tmpl_param->{value} || $tmpl_param->{value} eq '' ) {
            if ( $app->param('id') ) {
                $tmpl_param->{is_blank} = 1;
            } else {
                $tmpl_param->{value} = '{}';
            }
        }
    } elsif ( $tmpl_key eq 'options_field' ) {
        $tmpl_param->{options} ||= "{\n}";
    }
}

sub cb_template_param_asset_modal_fixture_mt62 {
    return 1 unless installed;
    my ( $cb, $app, $param, $tmpl ) = @_;

    # MT6.2の複数ファイルアップロードの動的なファイル・タイプ判定にて
    # files[i].type(ファイルのMIMEタイプ)としてm!^sheetasset_xls/.*!を必要としてしまう問題に対応
    # レジストリのsheetasset/types/*/type_of_mimeにMIMEのタイプ部分を記述し、
    # SheetAssetが定義するファイルの種類ではそれに差し替える
    my $require_type = $param->{require_type} || return 1;
    if ( $require_type =~ /^sheetasset_(.+)$/ ) {
        my $key = $1;
        if ( my $type = MT->registry('sheetasset', 'types', $key) ) {
            if ( defined(my $t = $type->{type_of_mime}) ) {
                $param->{require_type} = $t;
            }
        }
    }

    1;
}

1;
