package MT::SheetAsset::DataAPI;

use strict;
use warnings;

use MT::DataAPI::Endpoint::Asset;
use MT::DataAPI::Endpoint::Common;
use MT::DataAPI::Endpoint::v2::Tag;
use MT::DataAPI::Resource;
use MT::SheetAsset::Util;

sub asset {
    my ( $app, $endpoint ) = @_;
    my %args = $app->param_hash;
    my $type = $args{type};

    my $cache = $app->request->cache || {}; # Cache for tests
    my ( $blog, $asset ) = ( $cache->{sheetasset_blog}, $cache->{sheetasset_asset} );
    if ( !$blog || !$asset ) {
        ( $blog, $asset ) = context_objects(@_) or return;
    }

    run_permission_filter( $app, 'data_api_view_permission_filter',
        'asset', $asset->id, obj_promise($asset) )
        or return;

    return $app->error(400) unless $asset->can('sheetasset_workbook');

    my $wb = $asset->sheetasset_workbook;
    my $ws;
    if ( defined( my $worksheet_name = $args{worksheet} ) ) {
        $ws = $wb->find_worksheet($worksheet_name) or return $app->error(404);
    }
    if ( defined( my $worksheet_index = $args{worksheet_index} ) ) {
        $ws = $wb->worksheets->[$worksheet_index - 1] or return $app->error(404);
    }

    my ( $res, $index ) = ( {}, 1 );
    if ( $ws ) {
        # 単独ワークシート
        if ( $type eq 'table' ) {
            # テーブル
            $res = $ws->hashes(\%args);
        } else {
            # 行列
            $res = $ws->to_hash;
        }
    } else {
        # ワークブック全体
        if ( $type eq 'table' && $args{workbook} ) {
            # 全体のテーブル
            $res = { table => $wb->hashes(\%args) };
        } else {
            $res = {
                worksheets => [ map {
                    # 各シートのテーブルか行列
                    my $hash = $type eq 'table' ? {
                        name => $_->name,
                        table => $_->hashes(\%args),
                    } : $_->to_hash;
                    $hash->{index} = $index++;
                    $hash;
                } @{$wb->worksheets} ],
            };
        }
    }

    $res;
}

1;
