package MT::SheetAsset::Model::Workbook;

use strict;
use warnings;
use base qw( MT::ErrorHandler );

use MT::Util;
use MT::SheetAsset::Util;

sub new {
    my $package = shift;
    my $self = $package->SUPER::new(@_);
    my $opts = shift || {};

    my $worksheets = $opts->{worksheets};
    $worksheets = [] unless ref $worksheets eq 'ARRAY';

    $self->{worksheets} = [ grep { $_ } map {
        if ( UNIVERSAL::isa( $_, 'MT::SheetAsset::Model::Worksheet' ) ) {
            $_;
        } elsif ( ref $_ eq 'HASH' ) {
            MT::SheetAsset::Model::Worksheet->new($_);
        } else {
            undef;
        }
    } @$worksheets ];
    $self->{current_worksheet} = $self->{worksheets}->[0];

    $self->{filepath} = $opts->{filepath} || '';

    $self;
}

sub to_hash {
    my $self = shift;

    my %hash = (
        filepath => $self->{filepath} || '',
        worksheets => [ map {
            $_->to_hash
        } @{$self->worksheets} ],
    );

    \%hash;
}

sub filepath {
    my $self = shift;
    if ( @_ ) {
        $self->{filepath} = shift;
    }
    $self->{filepath};
}

sub worksheets { shift->{worksheets} ||= [] }

sub current_worksheet {
    my $self = shift;
    $self->{current_worksheet};
}

sub add_worksheet {
    my $self = shift;

    foreach my $ws ( @_ ) {
        $ws->workbook($self);
        $self->{current_worksheet} ||= $ws;
        push @{$self->{worksheets}}, $ws;
    }

    @_;
}

sub find_worksheet {
    my $self = shift;
    my ( $name ) = @_;
    $name = '' unless defined $name;

    my @worksheets = grep { $_->name eq $name } @{$self->worksheets};
    shift @worksheets;
}

sub hashes {
    my $self = shift;
    my @rows;
    foreach my $ws ( @{$self->worksheets} ) {
        my $hashes = $ws->hashes(@_);
        push @rows, @$hashes;
    }

    \@rows;
}

sub window {
    my $self = shift;
    my $opts = shift || {};

    if ( my $worksheets = $opts->{worksheets} ) {
        splice @{$self->{worksheets}}, $worksheets;
        $self->{current_worksheet} = $self->{worksheets}->[0];
    }

    foreach my $ws ( @{$self->worksheets} ) {
        $ws->window($opts);
    }

    $self;
}

1;