package MT::SheetAsset::Type::Base;

use strict;
use warnings;

use File::Spec;
use File::Basename;
use MT;
use MT::FileMgr;
use MT::Util;
use MT::SheetAsset::Model::Workbook;
use MT::SheetAsset::Model::Worksheet;
use MT::SheetAsset::Util;

our %DEFAULT_ENCODING = (
    ja => 'cp932',
);

sub readable_path {
    my $self = shift;
    my $path = $self->file_path;
    return $self->error(plugin->translate('File "[_1]" is not readable.', $path))
        if !-f $path or !-r $path;

    $path;
}

sub default_encoding {
    MT->config('SheetAssetDefaultEncoding')
        || $DEFAULT_ENCODING{MT->config->DefaultLanguage}
        || 'utf8';
}

sub image_width { 9999 }
sub image_height { 9999 }
sub can_create_thumbnail { 1 }
sub has_thumbnail { 1 }

sub is_sheetasset { 1 }

sub thumbnail_url {
    my $self = shift;
    my %param = @_;

    my $app = MT->instance;
    my $url = '//www.ideamans.com/_themes/mt-plugin-sheet-asset/spreadsheet-45.png';
    ( $url, $param{Width}, $param{Height} );
}

sub deserialize_workbook_json {
    my $package = shift;
    my ( $json ) = @_;

    my $hash; eval { $hash = MT::Util::from_json($json); };
    $hash = MT::Util::from_json($json);
    my $wb = MT::SheetAsset::Model::Workbook->new($hash)
        or return;

    $wb;
}

sub deserialize_workbook_json_file {
    my $package = shift;
    my ( $path ) = @_;

    my $fmgr = MT::FileMgr->new('Local');
    my $json = $fmgr->get_data( $path, 'output' ) || '{}';

    $package->deserialize_workbook_json( $json );
}

sub serialize_workbook_json {
    my $package = shift;
    my ( $workbook ) = @_;

    my $hash = $workbook->to_hash || {};
    MT::Util::to_json($hash);
}

sub serialize_workbook_json_file {
    my $package = shift;
    my ( $workbook, $path ) = @_;

    my $json = $package->serialize_workbook_json($workbook);
    my $fmgr = MT::FileMgr->new('Local');

    $fmgr->put_data( $json, $path );
}

sub sheetasset_from_file {
    my $package = shift;
    my ( $path ) = @_;
    my ( $basename, $dir, $ext ) = File::Basename::fileparse( $path, '\.[^\.]*' );

    my $class;
    if ( lc($ext) eq '.sheetasset' ) {
        require MT::SheetAsset::Type::Internal;
        $class = 'MT::SheetAsset::Type::Internal';
    } else {
        $class = MT::Asset->handler_for_file($path);
    }

    return unless $class->can('sheetasset_workbook');

    my $asset = $class->new;
    $asset->file_path($path);
    $asset->file_name($basename);

    $ext =~ s/^\.+//g;
    $asset->file_ext($ext);

    $asset;
}

1;