package MT::SheetAsset::Type::CSV;

use strict;
use warnings;
use Text::CSV::Encoded;

use base qw( MT::SheetAsset::Type::Base MT::Asset );
use MT::Blog;
use MT::Website;

use MT::SheetAsset::Util;

__PACKAGE__->install_properties(
    { class_type => 'sheetasset.csv', child_of => [ 'MT::Blog', 'MT::Website' ], } );

# List of supported file extensions (to aid the stock 'can_handle' method.)
sub extensions {
    my $pkg = shift;
    return $pkg->SUPER::extensions(
        [ qr/csv/i ]
    );
}

sub class_label {
    plugin->translate('CSV File');
}

sub class_label_plural {
    plugin->translate('CSV Files');
}

sub sheetasset_workbook {
    my $self = shift;
    my $opts = shift || {};

    my $path = $self->readable_path or return;
    my $encoding = $opts->{encoding} || $self->default_encoding;

    my $csv = Text::CSV::Encoded->new ({
        encoding_in => $encoding,
        encoding_out => "utf8",
    }) or return $self->error(plugin->translate('Failure to create Text::CSV::Encoded object.'));

    open my $fh, '<', $path
        or return $self->error(plugin->translate('Failure to open file "[_1]".', $path));

    my @rows;
    while ( my $row = $csv->getline($fh) ) {
        push @rows, $row;
    }
    close $fh;

    my $wb = MT::SheetAsset::Model::Workbook->new({ filepath => $path });
    my $ws = MT::SheetAsset::Model::Worksheet->new({ name => '', rows => \@rows });
    $wb->add_worksheet($ws);

    $wb;
}

1;